#include "RFM01.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////// COMMANDS /////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//  1: Configuration Setting Command: Frequency band, low battery detector, wake-up timer, crystal oscillator load capacitance, baseband filter bandwidth, clock output
// 100 (CMD) 01 (433 MHz) 00 (Low Battery Det Off) 1 (Crystal On During Sleep Mode) 0101 (x3-x0 Crystal Load Cap) 100 (Baseband Bandwidth) 0 (dc)
// 200 kHz Bandwidth
//uint16_t configSetCmd = 0b1000100101011000;
// 270 kHz Bandwidth
uint16_t configSetCmd = 0b1000100101010110;
// 340 kHz Bandwidth
//uint16_t configSetCmd = 0b1000100101010100;
// 400 kHz Bandwidth
//uint16_t configSetCmd = 0b1000100101010010;

//  2: Frequency Setting Command: Set the frequency of the local oscillator
// 1010 (CMD) 0110 0100 0000 (F) -> f0 = 10 MHz * 1 * (43 + 1600(F)/4000) = 434 MHz
// F = 96 to 3903
uint16_t carrierFreqCmd = 0b1010011001000000;
// F = 3000 -> 437,5 MHz
//uint16_t carrierFreqCmd = 0b1010101110111000;

//  3: Receiver Setting Command: Set VDI source, LNA gain, RSSI threshold
// 1100 0000 (CMD) 10 (Valid Data Indicator VDI Output) 00 (LNA Gain) 000 (RSSI Threshold) 0 (Enable Receiver Chain)
uint16_t disableReceiverCmd = 0b1100000010000000;
uint16_t enableReceiverCmd = 0b1100000010000001;

uint16_t enableReceiverWithRSSICmd = 0b1100000000000001;
uint16_t disableReceiverWithRSSICmd = 0b1100000000000000;

//  4: Wake-Up Timer Command: Wake-up time period
uint16_t wakeUpTimerCmd = 0;

//  5: Low Duty-Cycle Command: Set duty-cycle, enable low duty-cycle mode
uint16_t lowDutyCycleCmd = 0;

//  6: Low Battery Detector and Clock Divider Command: Set LBD threshold voltage and microcontroller clock division ratio
// 1100 0010 (CMD) 000 (d2-d0 Clk Output Frequency) 00000 (t4-t0 Low Battery Threshold Voltage) -> V = 2.25V + T*0.1V
uint16_t lowBatClockDivCmd = 0b1100001000000000;

//  7: AFC Control Command: Set AFC parameters
// 1100 0110 (CMD) 10 (Automatic Operation) 01 (Frequency Offset Limit) 1 (Strobe Edge) 0 (High accuracy mode) 1 (Output frequency offset register) 1 (Calculate offset frequency)
uint16_t afcControlCmd = 0b1100011010011011;
// No restriction for frequency correction
uint16_t afcControlNoRestCmd = 0b1100011010001011;

//  8: Data Filter Command: Set data filter type, clock recovery parameters
// 1100 0100 (CMD) 0 (Clk Recovery Auto Lock Control) 0 (Clk Recovery Lock Control) 1 (x) 01 (Filter Type) 010 (f2-f0 DQD Threshold Parameter)
uint16_t dataFilterCmd = 0b1100010010101010;

//  9: Data Rate Command: Bit rate
// 1100 1000 (CMD) 0 (cs) 100 0111 (R) -> BR = 10 MHz / 29 / (R+1) / (1+cs*7) = 10 MHz / 29 / (71+1) / (1+0*7) = 4,789 kbps
// Data rate = 600 bps to 115,2 kbps
uint16_t dataRateCmd = 0b1100100001000111;
//uint16_t dataRateCmd = 0b1100100001001010;

// 10: Output and FIFO Command: Set FIFO IT level, FIFO start control, FIFO enable and FIFO fill enable
// 1100 1110 (CMD) 1000 (f3-f0 FIFO Interrupt IT Level) 01 (FIFO Fill Start Condition) 0 (Enables FIFO Fill After Synchron Word Reception) 0 (Enables 16 bit Deep FIFO Mode)
//uint16_t outputFifoCmd = 0b1100111010000100;
//uint16_t enableFifoCmd = 0b1100111010000111;

uint16_t enableFifoAlwaysFillCmd = 0b1100111010001111;
uint16_t disableFifoCmd = 0b1100111010000100;
uint16_t enableFifoSyncWordFillCmd = 0b1100111010000111;
//uint16_t enableFifoWithSyncWordCmd = 0b1110 10000111;

uint16_t setFifoAlwaysFillCmd = 0b1100111010001101;

uint16_t enableFifoVDIFillCmd = 0b1100111010000001;
uint16_t disableFifoVDIFillCmd = 0b1100111010000000;

// 11: Reset Mode Command: Enable / disable sensitive reset
uint16_t resetModeCmd = 0;

// 12: Status Register Command: Read status information
uint16_t statusReadCmd = 0b0000000000000000;

void rfm01_init(void)
{
	RFM01_PORT = (1<<RFM01_nSEL) | (1<<RFM01_nFFS);			//	 Ausgang H
	RFM01_DDR = (1<<RFM01_SDI) | (1<<RFM01_SCK) | (1<<RFM01_nSEL) | (1<<RFM01_nFFS);	// Ausg�nge
/*
	rfm01_send_cmd(0x0000);						// Status read

	rfm01_send_cmd(0x8958);

	rfm01_send_cmd(0xA620);

	rfm01_send_cmd(0xC810);

	rfm01_send_cmd(0xC69B);

	rfm01_send_cmd(0xC42A);

	rfm01_send_cmd(0xC280);

	rfm01_send_cmd(0xCE88);

	rfm01_send_cmd(0xCE8B);

	rfm01_send_cmd(0xC081);						// Bandbreite
*/

	rfm01_send_cmd(statusReadCmd);						// Status read
	rfm01_send_cmd(carrierFreqCmd);
	rfm01_send_cmd(configSetCmd);
	rfm01_send_cmd(dataRateCmd);
	rfm01_send_cmd(afcControlCmd);
	//rfm01_send_cmd(afcControlNoRestCmd);
	rfm01_send_cmd(dataFilterCmd);
	rfm01_send_cmd(lowBatClockDivCmd);
	rfm01_send_cmd(enableFifoSyncWordFillCmd);
	rfm01_send_cmd(disableFifoCmd);
	//rfm01_send_cmd(enableFifoVDIFillCmd);
	//rfm01_send_cmd(disableFifoVDIFillCmd);
	rfm01_send_cmd(disableReceiverCmd);
	
}

void rfm01_send_cmd(unsigned int command)
{
	unsigned char bit;
	nSEL_LOW();

	for (bit=0; bit<16; bit++)
	{
		if (command & 1000000000000000)
		{
			SDI_HIGH();
		}
		else
		{
			SDI_LOW();
		}
		asm("nop");
		SCK_HIGH();
		asm("nop");
		asm("nop");
		SCK_LOW();
		command = (command << 1);				// n�chstes Bit nach oben
	}
	nSEL_HIGH();
}

unsigned char rfm01_read_fifo(char *data_buf, char len)
{

	unsigned char byte = 0, i, j;

	rfm01_send_cmd(disableFifoCmd);
	//rfm01_send_cmd(disableFifoVDIFillCmd);

	//rfm01_send_cmd(enableFifoAlwaysFillCmd);						// set FIFO mode
	rfm01_send_cmd(enableFifoSyncWordFillCmd);
	//rfm01_send_cmd(enableFifoVDIFillCmd);
	
	rfm01_send_cmd(enableReceiverCmd);						// Empf�nger ein
	//rfm01_send_cmd(enableReceiverWithRSSICmd);

	//rfm01_send_cmd(enableFifoWithSyncWordCmd);						// enable FIFO

	//rfm01_send_cmd(setFifoAlwaysFillCmd);
	//rfm01_send_cmd(enableFifoCmd);
	/*
	rfm01_send_cmd(0xC081);						// Empf�nger ein

	rfm01_send_cmd(0xCE85);						// set FIFO mode

	rfm01_send_cmd(0xCE87);
	*/
		
	SCK_LOW();
	SDI_LOW();

	for (i=0; i<len; i++)
	{
		nSEL_LOW();
		asm("nop");
		asm("nop");
		
		while (SDO_isLOW())	// warten bis SDO 1 -> FFIL
		{
			//LED_ON();
		}
		//_delay_ms(50);
		//LED_OFF();
		
		nFFS_LOW();							// Read FIFO ein (nFFS auf Low)

		byte = 0;

		for (j=0; j<8; j++)
		{
			byte = (byte << 1);					// eins h�her schieben
//			if (SDO_isHIGH() == (1<<RFM01_SDO))	// Bit 1?
      if ((RFM01_PIN & (1<<RFM01_SDO)) == (1<<RFM01_SDO)) // Bit 1?
			{
				byte = (byte | 0x01);				// ja
			}
			asm("nop");
			SCK_HIGH();
			asm("nop");
			asm("nop");
			SCK_LOW();
		}
		*data_buf++ = byte;
		nSEL_HIGH();
		nFFS_HIGH();
	}

	rfm01_send_cmd(disableReceiverCmd);						// Empf�nger aus
	//rfm01_send_cmd(disableReceiverWithRSSICmd);						// Empf�nger aus
	//rfm01_send_cmd(0xC080);						// Empf�nger aus
	return byte;

}

/*
unsigned int rfm01_read_status()
{
	unsigned int status = 0;
	unsigned char bit = 0;

	SCK_LOW();
	SDI_LOW();

	nSEL_LOW();

	for (bit=0; bit<16; bit++)
	{
		status = (status << 1);						// eins h�her schieben
		if (SDO_isHIGH() == (1<<RFM01_SDO))	// Bit 1?
		{
			status = (status | 0x0001);		// ja
		}
		asm("nop");
		SCK_HIGH();
		asm("nop");
		asm("nop");
		asm("nop");
		asm("nop");

		SCK_LOW();

	}
	nSEL_HIGH();

	return status;
}
*/


