/********************************************************************
RFM02-Funkmodul (Senden)
*********************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include "rfm02.h"
#include "uart.h"

#define DDR_BLED DDRC
#define BLED 0
#define DDR_LED DDRB
#define LED 5

#define LED_ON() PORTB |= (1<<LED)
#define LED_OFF() PORTB &= ~(1<<LED)
#define BLED_ON() PORTC |= (1<<BLED)
#define BLED_OFF() PORTC &= ~(1<<BLED)


void gpio_init()
{
	// Initialize On-Board LED on PB5
	LED_OFF();
	DDR_LED |= (1<<LED);

	BLED_OFF();
	DDR_BLED |= (1<<BLED);
}

void view_byte(uint8_t byte)
{
	unsigned wait = 500;
	for (int bit=0;bit<8;bit++)
	{
		LED_ON();
		if ((byte & 0b00000001)==0) BLED_OFF();
		else BLED_ON();
		_delay_ms(wait);
		LED_OFF();
		_delay_ms(wait);
		byte = byte>>1;
	}
	BLED_OFF();
}

void error()
{
	for (int i=0; i<20;i++)
	{
		LED_ON();
		_delay_ms(50);
		LED_OFF();
		_delay_ms(50);
	}
}

void heartbeat()
{
	uint8_t wait = 100;
	//LED_ON();
	BLED_ON();
	_delay_ms(wait);
	//LED_OFF();
	BLED_OFF();
	_delay_ms(wait);
	//LED_ON();
	BLED_ON();
	_delay_ms(wait);
	//LED_OFF();
	BLED_OFF();
	_delay_ms(wait);
}

int main(void)
{
	uint8_t send_buf[] = {'0','1','2','3','4','5','6','7','8','9'};
 
	uart_init();
  uart_send_text("Reset TX\r\n");
	gpio_init();
  uart_send_text("gpio-Init ok\r\n");
	rfm02_init();
  uart_send_text("rfm02_init ok\r\n");

	uint8_t myWord = 0b11001010;
	//uint16_t myWord = 0b11111111;
	//uint16_t myWord = 0b00000000;
	heartbeat();
	_delay_ms(500);
		
	while(1)
	{
		//heartbeat();	
		uint8_t i;
		LED_ON();
		rfm02_transmit_data(send_buf);
		//_delay_ms(50);
    uart_send_text("rfm02_transmit_data: ");
    for (i=0; i< 10; i++)
      {
        uart_send_byte_hex(send_buf[i]);
        uart_send_byte(' ');
      }  
    uart_send_newline();
    
		LED_OFF();
		_delay_ms(1000);
	};
}
