#include "rfm02.h"


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////// COMMANDS /////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//  1: Configuration Setting Command: Frequency band, micro controller clock output, crystal load, capacitance, frequency deviation
// 100 (CMD) 01 (433 MHz) 000 (1 MHz CLK) 1000 (Crystal Load Cap) 0001 (m2-m0)
// f_out = f0 � (-1)^BIT * (M + 1) * (30 kHz) = 434 MHz - (-1)^BIT * (2+1)*30kHZ = 434,090 MHz / 433,910 MHz with 180 kHZ bandwidth
uint16_t configSetCmd = 0b1000100010000010;

//  2: Power Management Command: Crystal oscillator, synthesizer, power amplifier, low battery
// 1100 0000 (CMD) 1 (Auto Crystal) 1 (Auto PA) 1 (Crystal) 0 (Synthesizer) 0 (PA) 0 (Low Bat Det) 0 (Wake Up Timer) 1 (Disable CLK out)
uint16_t standbyCmd = 0b1100000000000000;
//uint16_t powerManagementCmd = 0b1100000000111000;
uint16_t powerManagementAutoCmd = 0b1100000011100000;

//  3: Frequency Setting Command: Carrier frequency
// 1010 (CMD) 0110 0100 0000 (F) -> f0 = 10 MHz * 1 * (43 + 1600(F)/4000) = 434 MHz
// F = 96 to 3903
uint16_t carrierFreqCmd = 0b1010011001000000;
// F = 3000 -> 437,5 MHz
//uint16_t carrierFreqCmd = 0b1010101110111000;

//  4: Data Rate Command: Bit rate
// 1100 1000 (CMD) 0100 0111 (R) -> BR = 10 MHz / 29 / (R+1) = 10 MHz / 29 / (71+1) = 4,789 kbps
// 1100 1000 (CMD) 0000 1010 (R) -> BR = 10 MHz / 29 / (R+1) = 10 MHz / 29 / (10+1) = 31,347 kbps
// Data rate with minimal error = 2,4 kbps to 115,2 kbps
uint16_t dataRateCmd = 0b1100100001000111;
//uint16_t dataRateCmd = 0b1100100001001010;

//  5: Power Setting Command: Nominal output power, OOK mode
// 1011 (CMD) 0 (OOK Mode disabled) 000 (0 dBm Output Power)
uint8_t powerSetCmd = 0b10110000;

//  6: Low Battery Detector Command: Low battery threshold limit ACHTUNG DIESER BEFEHL SIEHT IM ORIGINAL DATENBLATT ANDERS AUS!!!
// 1100 0010 (CMD) 1 (dwc) 0 (none) 1 (ebs) 0 0000 (T)
// ebs enables the TX bit synchronization circuit and nIRQ can be used to determine start the next TX bit can be sended
uint16_t lowBatteryBitSyncCmd = 0b1100001010100000;

//  7: Sleep Command: Length of the clock tail after power down
uint16_t sleepCmd = 0;

//  8: Wake-Up Timer Command: Wake-up time period
uint16_t wakeUpTimerCmd = 0;

//  9: Data Transmit Command: Data transmission
uint8_t dataTransmitCmd = 0b11000110;

// 10: Status Register Command: Transmitter status read
uint16_t statusReadCmd = 0b1100110000000000;

// 11: PLL Setting Command: PLL bandwidth can be modified by this command
// D240: 25% current
uint16_t pllSetCmd = 0xD240;
// D2C0: 33% current
//uint16_t pllSetCmd = 0xD2C0;
// D280: 100% current
//uint16_t pllSetCmd = 0xD280;

void rfm02_init()
{
	RFM02_PORT |= (1<<RFM02_nIRQ) | (1<<RFM02_nSEL) | (1<<RFM02_SDI);    // PullUp ein / Ausgang H
	RFM02_DDR  |= (1<<RFM02_nSEL) | (1<<RFM02_SCK) | (1<<RFM02_SDI);    // Ausg�nge 

	RFM02_DDR  &= ~(1<<RFM02_nIRQ);                                    // Eingang

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////// Configure RF02 ///////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
  rfm02_send_cmd(0xCC00);   // Status read

  rfm02_send_cmd(0xA620);	  // Frequenz: 430MHz + (0,0025 * 0x620) -> 433,92MHz
  rfm02_send_cmd(0xC0E0);   // Power Settings: Quarz ein, Synthesizer und PA ein bei Send-Command
  rfm02_send_cmd(0xC2A0);	  // low bat detector, TX Bit Sync, WakUp-Timer Calibrierung aus
  rfm02_send_cmd(0x8883);	  // Control: 1MHz Clock Out, 12,5pF Last Quarz, 120kHz Shift
  //rfm02_send_cmd(0xA620);	  // Frequenz: 430MHz + (0,0025 * 0x620) -> 433,92MHz
  rfm02_send_cmd(0xB200);   // Power 2 * -3dBm -> -6dBm
  rfm02_send_cmd(0xD2C0);   // Set PLL: set 33%
  rfm02_send_cmd(0xC810);   // Baudrate: 344827 / 19200 = 17 - 1 -> 0x10                   // Status read
  rfm02_send_cmd(0xE500 + 200); // WakeUp-Timer 2^7 *234ms ~ 30s ???!! nicht 1ms sondern 10ms !!???
  rfm02_send_cmd(0xC002);   // alles aus, WakeUp-Timer ein

  rfm02_send_cmd(0xC440);   // sleep, noch 64 Takte
*/

	rfm02_send_cmd(statusReadCmd);
	rfm02_send_cmd(carrierFreqCmd);
	rfm02_send_cmd(configSetCmd);
	rfm02_send_cmd(dataRateCmd);
	rfm02_send_cmd(pllSetCmd);
	//rfm02_send_cmd(powerManagementAutoCmd);
	rfm02_send_cmd(lowBatteryBitSyncCmd);
	//nSEL_LOW();
	//rfm02_send_8bit(powerSetCmd);
	//nSEL_HIGH();
	rfm02_send_cmd(standbyCmd);
	

}

//-----------------------------------------------------------------------------------------

uint8_t rfm02_send_cmd(uint16_t command)
{
	uint8_t status;

	// nSEL goes Low
	nSEL_LOW();

	// Send higher byte
	rfm02_send_8bit(command >> 8);
	// Send lower byte
	status = rfm02_send_8bit(command & 0b11111111);

	nSEL_HIGH();

	return status;
}

//-----------------------------------------------------------------------------------------

uint8_t rfm02_send_8bit(uint8_t byte)
{
	uint8_t bit;

	for (bit=0; bit<8; bit++)
	{
		if (byte & 0b10000000)
		{
			SDI_HIGH();
		}
		else
		{
			SDI_LOW();
		}
		
		asm("nop");
		SCK_HIGH();
		asm("nop");
		asm("nop");

		byte = (byte << 1);			            	// n�chstes Bit nach oben

		if (nIRQ_isHIGH())       // unteres Bit Status einlesen, H?
		{
			byte |= 0x01;                     // DATA war H
		}
		else
		{
			byte &= ~(0x01);               		// DATA war L
		}
		asm("nop");
		SCK_LOW();
	}
		
	return byte;
}

//-----------------------------------------------------------------------------------------

void rfm02_send_data(uint8_t byte)
{

	uint8_t bit;

	for (bit=0; bit<8; bit++)
	{
		while (!(RFM02_PIN & (1<<RFM02_nIRQ))) // warten bis IRQ L/H
		{
		}

		while (RFM02_PIN & (1<<RFM02_nIRQ))    // warten bis IRQ Impuls zuende H/L
		{
		}

		if (byte & 0b10000000)
		{
			SDI_HIGH();
		}
		else
		{
			SDI_LOW();
		}
		byte = (byte << 1);                   // Bit 7 rausschieben
	}
}

//-----------------------------------------------------------------------------------------

void rfm02_transmit_data(uint8_t data[])
{
	uint8_t it;

	rfm02_send_cmd(powerManagementAutoCmd);                   // einschalten
	//rfm02_send_cmd(0xC0E0);                   // einschalten
	
	nSEL_LOW();

	rfm02_send_8bit(dataTransmitCmd);                    // Kommando Daten senden
	//rfm02_send_8bit(0xC6);                    // Kommando Daten senden
	
	rfm02_send_data(0xAA);                    // Sync
	rfm02_send_data(0xAA);
	rfm02_send_data(0xAA);
	// RFM01 Sync Word: 2DD4
	rfm02_send_data(0x2D);                    // Magic
	rfm02_send_data(0xD4);

  int i;
	// My word!
	for (i=0; i<10;i++)
	{
		rfm02_send_data(data[i]);
	}

	while (nIRQ_isLOW())     // auf Ende sendes letztes Byte warten -> L/H
	{
	}
	while (nIRQ_isHIGH())        // erledigt, wenn IRQ wieder -> H/L
	{
	}
	while (nIRQ_isLOW())     // auf Ende Power Off warten -> L/H
	{
	}

	nSEL_HIGH();

	rfm02_send_cmd(standbyCmd); 						      // alles aus, WakeUp-Timer ein
	//rfm02_send_cmd(0xC002); 						      // alles aus, WakeUp-Timer ein
	//rfm02_send_cmd(0xC440);     						  // sleep, noch 64 Takte


}
